@@ -1,5 +1,7 @@
-from . import testing, tutorial
-from .backends.api import (
+from importlib.metadata import version as _version
+
+from xarray import testing, tutorial
+from xarray.backends.api import (
     load_dataarray,
     load_dataset,
     open_dataarray,
@@ -7,16 +9,15 @@ from .backends.api import (
     open_mfdataset,
     save_mfdataset,
 )
-from .backends.rasterio_ import open_rasterio
-from .backends.zarr import open_zarr
-from .coding.cftime_offsets import cftime_range, date_range, date_range_like
-from .coding.cftimeindex import CFTimeIndex
-from .coding.frequencies import infer_freq
-from .conventions import SerializationWarning, decode_cf
-from .core.alignment import align, broadcast
-from .core.combine import combine_by_coords, combine_nested
-from .core.common import ALL_DIMS, full_like, ones_like, zeros_like
-from .core.computation import (
+from xarray.backends.zarr import open_zarr
+from xarray.coding.cftime_offsets import cftime_range, date_range, date_range_like
+from xarray.coding.cftimeindex import CFTimeIndex
+from xarray.coding.frequencies import infer_freq
+from xarray.conventions import SerializationWarning, decode_cf
+from xarray.core.alignment import align, broadcast
+from xarray.core.combine import combine_by_coords, combine_nested
+from xarray.core.common import ALL_DIMS, full_like, ones_like, zeros_like
+from xarray.core.computation import (
     apply_ufunc,
     corr,
     cov,
@@ -26,28 +27,28 @@ from .core.computation import (
     unify_chunks,
     where,
 )
-from .core.concat import concat
-from .core.dataarray import DataArray
-from .core.dataset import Dataset
-from .core.extensions import register_dataarray_accessor, register_dataset_accessor
-from .core.merge import Context, MergeError, merge
-from .core.options import get_options, set_options
-from .core.parallel import map_blocks
-from .core.variable import Coordinate, IndexVariable, Variable, as_variable
-from .util.print_versions import show_versions
-
-try:
-    from importlib.metadata import version as _version
-except ImportError:
-    # if the fallback library is missing, we are doomed.
-    from importlib_metadata import version as _version
+from xarray.core.concat import concat
+from xarray.core.coordinates import Coordinates
+from xarray.core.dataarray import DataArray
+from xarray.core.dataset import Dataset
+from xarray.core.extensions import (
+    register_dataarray_accessor,
+    register_dataset_accessor,
+)
+from xarray.core.indexes import Index
+from xarray.core.indexing import IndexSelResult
+from xarray.core.merge import Context, MergeError, merge
+from xarray.core.options import get_options, set_options
+from xarray.core.parallel import map_blocks
+from xarray.core.variable import IndexVariable, Variable, as_variable
+from xarray.util.print_versions import show_versions
 
 try:
     __version__ = _version("xarray")
 except Exception:
     # Local copy or not installed with setuptools.
     # Disable minimum version checks on downstream libraries.
-    __version__ = "999"
+    __version__ = "9999"
 
 # A hardcoded __all__ variable is necessary to appease
 # `mypy --strict` running in projects that import xarray.
@@ -82,7 +83,6 @@ __all__ = (
     "open_dataarray",
     "open_dataset",
     "open_mfdataset",
-    "open_rasterio",
     "open_zarr",
     "polyval",
     "register_dataarray_accessor",
@@ -97,8 +97,11 @@ __all__ = (
     "CFTimeIndex",
     "Context",
     "Coordinate",
+    "Coordinates",
     "DataArray",
     "Dataset",
+    "Index",
+    "IndexSelResult",
     "IndexVariable",
     "Variable",
     # Exceptions
