@@ -3,6 +3,8 @@ from __future__ import annotations
 import gzip
 import io
 import os
+from collections.abc import Iterable
+from typing import TYPE_CHECKING, Any
 
 import numpy as np
 
@@ -26,11 +28,16 @@ from xarray.core.utils import (
     Frozen,
     FrozenDict,
     close_on_error,
-    module_available,
     try_read_magic_number_from_file_or_path,
 )
 from xarray.core.variable import Variable
 
+if TYPE_CHECKING:
+    from io import BufferedIOBase
+
+    from xarray.backends.common import AbstractDataStore
+    from xarray.core.dataset import Dataset
+
 
 def _decode_string(s):
     if isinstance(s, bytes):
@@ -261,33 +268,35 @@ class ScipyBackendEntrypoint(BackendEntrypoint):
     backends.H5netcdfBackendEntrypoint
     """
 
-    available = module_available("scipy")
     description = "Open netCDF files (.nc, .nc4, .cdf and .gz) using scipy in Xarray"
     url = "https://docs.xarray.dev/en/stable/generated/xarray.backends.ScipyBackendEntrypoint.html"
 
-    def guess_can_open(self, filename_or_obj):
-
+    def guess_can_open(
+        self,
+        filename_or_obj: str | os.PathLike[Any] | BufferedIOBase | AbstractDataStore,
+    ) -> bool:
         magic_number = try_read_magic_number_from_file_or_path(filename_or_obj)
         if magic_number is not None and magic_number.startswith(b"\x1f\x8b"):
-            with gzip.open(filename_or_obj) as f:
+            with gzip.open(filename_or_obj) as f:  # type: ignore[arg-type]
                 magic_number = try_read_magic_number_from_file_or_path(f)
         if magic_number is not None:
             return magic_number.startswith(b"CDF")
 
-        try:
+        if isinstance(filename_or_obj, (str, os.PathLike)):
             _, ext = os.path.splitext(filename_or_obj)
-        except TypeError:
-            return False
-        return ext in {".nc", ".nc4", ".cdf", ".gz"}
+            return ext in {".nc", ".nc4", ".cdf", ".gz"}
 
-    def open_dataset(
+        return False
+
+    def open_dataset(  # type: ignore[override]  # allow LSP violation, not supporting **kwargs
         self,
-        filename_or_obj,
+        filename_or_obj: str | os.PathLike[Any] | BufferedIOBase | AbstractDataStore,
+        *,
         mask_and_scale=True,
         decode_times=True,
         concat_characters=True,
         decode_coords=True,
-        drop_variables=None,
+        drop_variables: str | Iterable[str] | None = None,
         use_cftime=None,
         decode_timedelta=None,
         mode="r",
@@ -295,8 +304,7 @@ class ScipyBackendEntrypoint(BackendEntrypoint):
         group=None,
         mmap=None,
         lock=None,
-    ):
-
+    ) -> Dataset:
         filename_or_obj = _normalize_path(filename_or_obj)
         store = ScipyDataStore(
             filename_or_obj, mode=mode, format=format, group=group, mmap=mmap, lock=lock
@@ -317,4 +325,4 @@ class ScipyBackendEntrypoint(BackendEntrypoint):
         return ds
 
 
-BACKEND_ENTRYPOINTS["scipy"] = ScipyBackendEntrypoint
+BACKEND_ENTRYPOINTS["scipy"] = ("scipy", ScipyBackendEntrypoint)
