@@ -6,17 +6,21 @@ import numbers
 import numpy as np
 
 # _typed_ops.py is a generated file
-from ._typed_ops import (
+from xarray.core._typed_ops import (
     DataArrayGroupByOpsMixin,
     DataArrayOpsMixin,
     DatasetGroupByOpsMixin,
     DatasetOpsMixin,
     VariableOpsMixin,
 )
-from .common import ImplementsArrayReduce, ImplementsDatasetReduce
-from .ops import IncludeCumMethods, IncludeNumpySameMethods, IncludeReduceMethods
-from .options import OPTIONS, _get_keep_attrs
-from .pycompat import is_duck_array
+from xarray.core.common import ImplementsArrayReduce, ImplementsDatasetReduce
+from xarray.core.ops import (
+    IncludeCumMethods,
+    IncludeNumpySameMethods,
+    IncludeReduceMethods,
+)
+from xarray.core.options import OPTIONS, _get_keep_attrs
+from xarray.core.pycompat import is_duck_array
 
 
 class SupportsArithmetic:
@@ -40,7 +44,7 @@ class SupportsArithmetic:
     )
 
     def __array_ufunc__(self, ufunc, method, *inputs, **kwargs):
-        from .computation import apply_ufunc
+        from xarray.core.computation import apply_ufunc
 
         # See the docstring example for numpy.lib.mixins.NDArrayOperatorsMixin.
         out = kwargs.get("out", ())
@@ -52,10 +56,10 @@ class SupportsArithmetic:
 
         if ufunc.signature is not None:
             raise NotImplementedError(
-                "{} not supported: xarray objects do not directly implement "
+                f"{ufunc} not supported: xarray objects do not directly implement "
                 "generalized ufuncs. Instead, use xarray.apply_ufunc or "
                 "explicitly convert to xarray objects to NumPy arrays "
-                "(e.g., with `.values`).".format(ufunc)
+                "(e.g., with `.values`)."
             )
 
         if method != "__call__":
