@@ -3,39 +3,59 @@ from __future__ import annotations
 import collections.abc
 import copy
 from collections import defaultdict
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Dict,
-    Generic,
-    Hashable,
-    Iterable,
-    Iterator,
-    Mapping,
-    Sequence,
-    TypeVar,
-    cast,
-)
+from collections.abc import Hashable, Iterable, Iterator, Mapping, Sequence
+from typing import TYPE_CHECKING, Any, Generic, TypeVar, cast
 
 import numpy as np
 import pandas as pd
 
-from . import formatting, nputils, utils
-from .indexing import IndexSelResult, PandasIndexingAdapter, PandasMultiIndexingAdapter
-from .utils import Frozen, get_valid_numpy_dtype, is_dict_like, is_scalar
+from xarray.core import formatting, nputils, utils
+from xarray.core.indexing import (
+    IndexSelResult,
+    PandasIndexingAdapter,
+    PandasMultiIndexingAdapter,
+)
+from xarray.core.utils import (
+    Frozen,
+    emit_user_level_warning,
+    get_valid_numpy_dtype,
+    is_dict_like,
+    is_scalar,
+)
 
 if TYPE_CHECKING:
-    from .types import ErrorOptions, T_Index
-    from .variable import Variable
+    from xarray.core.types import ErrorOptions, JoinOptions, Self
+    from xarray.core.variable import Variable
 
-IndexVars = Dict[Any, "Variable"]
+
+IndexVars = dict[Any, "Variable"]
 
 
 class Index:
-    """Base class inherited by all xarray-compatible indexes.
+    """
+    Base class inherited by all xarray-compatible indexes.
+
+    Do not use this class directly for creating index objects. Xarray indexes
+    are created exclusively from subclasses of ``Index``, mostly via Xarray's
+    public API like ``Dataset.set_xindex``.
+
+    Every subclass must at least implement :py:meth:`Index.from_variables`. The
+    (re)implementation of the other methods of this base class is optional but
+    mostly required in order to support operations relying on indexes such as
+    label-based selection or alignment.
+
+    The ``Index`` API closely follows the :py:meth:`Dataset` and
+    :py:meth:`DataArray` API, e.g., for an index to support ``.sel()`` it needs
+    to implement :py:meth:`Index.sel`, to support ``.stack()`` and
+    ``.unstack()`` it needs to implement :py:meth:`Index.stack` and
+    :py:meth:`Index.unstack`, etc.
+
+    When a method is not (re)implemented, depending on the case the
+    corresponding operation on a :py:meth:`Dataset` or :py:meth:`DataArray`
+    either will raise a ``NotImplementedError`` or will simply drop/pass/copy
+    the index from/to the result.
 
     Do not use this class directly for creating index objects.
-
     """
 
     @classmethod
@@ -44,30 +64,129 @@ class Index:
         variables: Mapping[Any, Variable],
         *,
         options: Mapping[str, Any],
-    ) -> Index:
+    ) -> Self:
+        """Create a new index object from one or more coordinate variables.
+
+        This factory method must be implemented in all subclasses of Index.
+
+        The coordinate variables may be passed here in an arbitrary number and
+        order and each with arbitrary dimensions. It is the responsibility of
+        the index to check the consistency and validity of these coordinates.
+
+        Parameters
+        ----------
+        variables : dict-like
+            Mapping of :py:class:`Variable` objects holding the coordinate labels
+            to index.
+
+        Returns
+        -------
+        index : Index
+            A new Index object.
+        """
         raise NotImplementedError()
 
     @classmethod
     def concat(
-        cls: type[T_Index],
-        indexes: Sequence[T_Index],
+        cls,
+        indexes: Sequence[Self],
         dim: Hashable,
         positions: Iterable[Iterable[int]] | None = None,
-    ) -> T_Index:
+    ) -> Self:
+        """Create a new index by concatenating one or more indexes of the same
+        type.
+
+        Implementation is optional but required in order to support
+        ``concat``. Otherwise it will raise an error if the index needs to be
+        updated during the operation.
+
+        Parameters
+        ----------
+        indexes : sequence of Index objects
+            Indexes objects to concatenate together. All objects must be of the
+            same type.
+        dim : Hashable
+            Name of the dimension to concatenate along.
+        positions : None or list of integer arrays, optional
+            List of integer arrays which specifies the integer positions to which
+            to assign each dataset along the concatenated dimension. If not
+            supplied, objects are concatenated in the provided order.
+
+        Returns
+        -------
+        index : Index
+            A new Index object.
+        """
         raise NotImplementedError()
 
     @classmethod
-    def stack(cls, variables: Mapping[Any, Variable], dim: Hashable) -> Index:
+    def stack(cls, variables: Mapping[Any, Variable], dim: Hashable) -> Self:
+        """Create a new index by stacking coordinate variables into a single new
+        dimension.
+
+        Implementation is optional but required in order to support ``stack``.
+        Otherwise it will raise an error when trying to pass the Index subclass
+        as argument to :py:meth:`Dataset.stack`.
+
+        Parameters
+        ----------
+        variables : dict-like
+            Mapping of :py:class:`Variable` objects to stack together.
+        dim : Hashable
+            Name of the new, stacked dimension.
+
+        Returns
+        -------
+        index
+            A new Index object.
+        """
         raise NotImplementedError(
             f"{cls!r} cannot be used for creating an index of stacked coordinates"
         )
 
     def unstack(self) -> tuple[dict[Hashable, Index], pd.MultiIndex]:
+        """Unstack a (multi-)index into multiple (single) indexes.
+
+        Implementation is optional but required in order to support unstacking
+        the coordinates from which this index has been built.
+
+        Returns
+        -------
+        indexes : tuple
+            A 2-length tuple where the 1st item is a dictionary of unstacked
+            Index objects and the 2nd item is a :py:class:`pandas.MultiIndex`
+            object used to unstack unindexed coordinate variables or data
+            variables.
+        """
         raise NotImplementedError()
 
     def create_variables(
         self, variables: Mapping[Any, Variable] | None = None
     ) -> IndexVars:
+        """Maybe create new coordinate variables from this index.
+
+        This method is useful if the index data can be reused as coordinate
+        variable data. It is often the case when the underlying index structure
+        has an array-like interface, like :py:class:`pandas.Index` objects.
+
+        The variables given as argument (if any) are either returned as-is
+        (default behavior) or can be used to copy their metadata (attributes and
+        encoding) into the new returned coordinate variables.
+
+        Note: the input variables may or may not have been filtered for this
+        index.
+
+        Parameters
+        ----------
+        variables : dict-like, optional
+            Mapping of :py:class:`Variable` objects.
+
+        Returns
+        -------
+        index_variables : dict-like
+            Dictionary of :py:class:`Variable` or :py:class:`IndexVariable`
+            objects.
+        """
         if variables is not None:
             # pass through
             return dict(**variables)
@@ -75,54 +194,213 @@ class Index:
             return {}
 
     def to_pandas_index(self) -> pd.Index:
-        """Cast this xarray index to a pandas.Index object or raise a TypeError
-        if this is not supported.
+        """Cast this xarray index to a pandas.Index object or raise a
+        ``TypeError`` if this is not supported.
 
-        This method is used by all xarray operations that expect/require a
-        pandas.Index object.
+        This method is used by all xarray operations that still rely on
+        pandas.Index objects.
 
+        By default it raises a ``TypeError``, unless it is re-implemented in
+        subclasses of Index.
         """
         raise TypeError(f"{self!r} cannot be cast to a pandas.Index object")
 
     def isel(
         self, indexers: Mapping[Any, int | slice | np.ndarray | Variable]
-    ) -> Index | None:
+    ) -> Self | None:
+        """Maybe returns a new index from the current index itself indexed by
+        positional indexers.
+
+        This method should be re-implemented in subclasses of Index if the
+        wrapped index structure supports indexing operations. For example,
+        indexing a ``pandas.Index`` is pretty straightforward as it behaves very
+        much like an array. By contrast, it may be harder doing so for a
+        structure like a kd-tree that differs much from a simple array.
+
+        If not re-implemented in subclasses of Index, this method returns
+        ``None``, i.e., calling :py:meth:`Dataset.isel` will either drop the
+        index in the resulting dataset or pass it unchanged if its corresponding
+        coordinate(s) are not indexed.
+
+        Parameters
+        ----------
+        indexers : dict
+            A dictionary of positional indexers as passed from
+            :py:meth:`Dataset.isel` and where the entries have been filtered
+            for the current index.
+
+        Returns
+        -------
+        maybe_index : Index
+            A new Index object or ``None``.
+        """
         return None
 
     def sel(self, labels: dict[Any, Any]) -> IndexSelResult:
+        """Query the index with arbitrary coordinate label indexers.
+
+        Implementation is optional but required in order to support label-based
+        selection. Otherwise it will raise an error when trying to call
+        :py:meth:`Dataset.sel` with labels for this index coordinates.
+
+        Coordinate label indexers can be of many kinds, e.g., scalar, list,
+        tuple, array-like, slice, :py:class:`Variable`, :py:class:`DataArray`, etc.
+        It is the responsibility of the index to handle those indexers properly.
+
+        Parameters
+        ----------
+        labels : dict
+            A dictionary of coordinate label indexers passed from
+            :py:meth:`Dataset.sel` and where the entries have been filtered
+            for the current index.
+
+        Returns
+        -------
+        sel_results : :py:class:`IndexSelResult`
+            An index query result object that contains dimension positional indexers.
+            It may also contain new indexes, coordinate variables, etc.
+        """
         raise NotImplementedError(f"{self!r} doesn't support label-based selection")
 
-    def join(self: T_Index, other: T_Index, how: str = "inner") -> T_Index:
+    def join(self, other: Self, how: JoinOptions = "inner") -> Self:
+        """Return a new index from the combination of this index with another
+        index of the same type.
+
+        Implementation is optional but required in order to support alignment.
+
+        Parameters
+        ----------
+        other : Index
+            The other Index object to combine with this index.
+        join : str, optional
+            Method for joining the two indexes (see :py:func:`~xarray.align`).
+
+        Returns
+        -------
+        joined : Index
+            A new Index object.
+        """
         raise NotImplementedError(
             f"{self!r} doesn't support alignment with inner/outer join method"
         )
 
-    def reindex_like(self: T_Index, other: T_Index) -> dict[Hashable, Any]:
+    def reindex_like(self, other: Self) -> dict[Hashable, Any]:
+        """Query the index with another index of the same type.
+
+        Implementation is optional but required in order to support alignment.
+
+        Parameters
+        ----------
+        other : Index
+            The other Index object used to query this index.
+
+        Returns
+        -------
+        dim_positional_indexers : dict
+            A dictionary where keys are dimension names and values are positional
+            indexers.
+        """
         raise NotImplementedError(f"{self!r} doesn't support re-indexing labels")
 
-    def equals(self, other):  # pragma: no cover
+    def equals(self, other: Self) -> bool:
+        """Compare this index with another index of the same type.
+
+        Implementation is optional but required in order to support alignment.
+
+        Parameters
+        ----------
+        other : Index
+            The other Index object to compare with this object.
+
+        Returns
+        -------
+        is_equal : bool
+            ``True`` if the indexes are equal, ``False`` otherwise.
+        """
         raise NotImplementedError()
 
-    def roll(self, shifts: Mapping[Any, int]) -> Index | None:
+    def roll(self, shifts: Mapping[Any, int]) -> Self | None:
+        """Roll this index by an offset along one or more dimensions.
+
+        This method can be re-implemented in subclasses of Index, e.g., when the
+        index can be itself indexed.
+
+        If not re-implemented, this method returns ``None``, i.e., calling
+        :py:meth:`Dataset.roll` will either drop the index in the resulting
+        dataset or pass it unchanged if its corresponding coordinate(s) are not
+        rolled.
+
+        Parameters
+        ----------
+        shifts : mapping of hashable to int, optional
+            A dict with keys matching dimensions and values given
+            by integers to rotate each of the given dimensions, as passed
+            :py:meth:`Dataset.roll`.
+
+        Returns
+        -------
+        rolled : Index
+            A new index with rolled data.
+        """
         return None
 
     def rename(
-        self, name_dict: Mapping[Any, Hashable], dims_dict: Mapping[Any, Hashable]
-    ) -> Index:
+        self,
+        name_dict: Mapping[Any, Hashable],
+        dims_dict: Mapping[Any, Hashable],
+    ) -> Self:
+        """Maybe update the index with new coordinate and dimension names.
+
+        This method should be re-implemented in subclasses of Index if it has
+        attributes that depend on coordinate or dimension names.
+
+        By default (if not re-implemented), it returns the index itself.
+
+        Warning: the input names are not filtered for this method, they may
+        correspond to any variable or dimension of a Dataset or a DataArray.
+
+        Parameters
+        ----------
+        name_dict : dict-like
+            Mapping of current variable or coordinate names to the desired names,
+            as passed from :py:meth:`Dataset.rename_vars`.
+        dims_dict : dict-like
+            Mapping of current dimension names to the desired names, as passed
+            from :py:meth:`Dataset.rename_dims`.
+
+        Returns
+        -------
+        renamed : Index
+            Index with renamed attributes.
+        """
         return self
 
-    def __copy__(self) -> Index:
-        return self._copy(deep=False)
+    def copy(self, deep: bool = True) -> Self:
+        """Return a (deep) copy of this index.
+
+        Implementation in subclasses of Index is optional. The base class
+        implements the default (deep) copy semantics.
+
+        Parameters
+        ----------
+        deep : bool, optional
+            If true (default), a copy of the internal structures
+            (e.g., wrapped index) is returned with the new object.
+
+        Returns
+        -------
+        index : Index
+            A new Index object.
+        """
+        return self._copy(deep=deep)
+
+    def __copy__(self) -> Self:
+        return self.copy(deep=False)
 
     def __deepcopy__(self, memo: dict[int, Any] | None = None) -> Index:
         return self._copy(deep=True, memo=memo)
 
-    def copy(self: T_Index, deep: bool = True) -> T_Index:
-        return self._copy(deep=deep)
-
-    def _copy(
-        self: T_Index, deep: bool = True, memo: dict[int, Any] | None = None
-    ) -> T_Index:
+    def _copy(self, deep: bool = True, memo: dict[int, Any] | None = None) -> Self:
         cls = self.__class__
         copied = cls.__new__(cls)
         if deep:
@@ -132,7 +410,7 @@ class Index:
             copied.__dict__.update(self.__dict__)
         return copied
 
-    def __getitem__(self, indexer: Any):
+    def __getitem__(self, indexer: Any) -> Self:
         raise NotImplementedError()
 
     def _repr_inline_(self, max_width):
@@ -140,9 +418,9 @@ class Index:
 
 
 def _maybe_cast_to_cftimeindex(index: pd.Index) -> pd.Index:
-    from ..coding.cftimeindex import CFTimeIndex
+    from xarray.coding.cftimeindex import CFTimeIndex
 
-    if len(index) > 0 and index.dtype == "O":
+    if len(index) > 0 and index.dtype == "O" and not isinstance(index, CFTimeIndex):
         try:
             return CFTimeIndex(index)
         except (ImportError, TypeError):
@@ -160,8 +438,8 @@ def safe_cast_to_index(array: Any) -> pd.Index:
     this function will not attempt to do automatic type conversion but will
     always return an index with dtype=object.
     """
-    from .dataarray import DataArray
-    from .variable import Variable
+    from xarray.core.dataarray import DataArray
+    from xarray.core.variable import Variable
 
     if isinstance(array, pd.Index):
         index = array
@@ -173,17 +451,29 @@ def safe_cast_to_index(array: Any) -> pd.Index:
     elif isinstance(array, PandasIndexingAdapter):
         index = array.array
     else:
-        kwargs = {}
-        if hasattr(array, "dtype") and array.dtype.kind == "O":
-            kwargs["dtype"] = object
+        kwargs: dict[str, str] = {}
+        if hasattr(array, "dtype"):
+            if array.dtype.kind == "O":
+                kwargs["dtype"] = "object"
+            elif array.dtype == "float16":
+                emit_user_level_warning(
+                    (
+                        "`pandas.Index` does not support the `float16` dtype."
+                        " Casting to `float64` for you, but in the future please"
+                        " manually cast to either `float32` and `float64`."
+                    ),
+                    category=DeprecationWarning,
+                )
+                kwargs["dtype"] = "float64"
+
         index = pd.Index(np.asarray(array), **kwargs)
 
     return _maybe_cast_to_cftimeindex(index)
 
 
 def _sanitize_slice_element(x):
-    from .dataarray import DataArray
-    from .variable import Variable
+    from xarray.core.dataarray import DataArray
+    from xarray.core.variable import Variable
 
     if not isinstance(x, tuple) and len(np.shape(x)) != 0:
         raise ValueError(
@@ -266,6 +556,8 @@ def get_indexer_nd(index, labels, method=None, tolerance=None):
     labels
     """
     flat_labels = np.ravel(labels)
+    if flat_labels.dtype == "float16":
+        flat_labels = flat_labels.astype("float64")
     flat_indexer = index.get_indexer(flat_labels, method=method, tolerance=tolerance)
     indexer = flat_indexer.reshape(labels.shape)
     return indexer
@@ -320,7 +612,14 @@ class PandasIndex(Index):
 
         name, var = next(iter(variables.items()))
 
-        if var.ndim != 1:
+        if var.ndim == 0:
+            raise ValueError(
+                f"cannot set a PandasIndex from the scalar variable {name!r}, "
+                "only 1-dimensional variables are supported. "
+                f"Note: you might want to use `obj.expand_dims({name!r})` to create a "
+                f"new dimension and turn {name!r} as an indexed dimension coordinate."
+            )
+        elif var.ndim != 1:
             raise ValueError(
                 "PandasIndex only accepts a 1-dimensional variable, "
                 f"variable {name!r} has {var.ndim} dimensions"
@@ -371,10 +670,10 @@ class PandasIndex(Index):
     @classmethod
     def concat(
         cls,
-        indexes: Sequence[PandasIndex],
+        indexes: Sequence[Self],
         dim: Hashable,
         positions: Iterable[Iterable[int]] | None = None,
-    ) -> PandasIndex:
+    ) -> Self:
         new_pd_index = cls._concat_indexes(indexes, dim, positions)
 
         if not indexes:
@@ -387,7 +686,7 @@ class PandasIndex(Index):
     def create_variables(
         self, variables: Mapping[Any, Variable] | None = None
     ) -> IndexVars:
-        from .variable import IndexVariable
+        from xarray.core.variable import IndexVariable
 
         name = self.index.name
         attrs: Mapping[Hashable, Any] | None
@@ -411,7 +710,7 @@ class PandasIndex(Index):
     def isel(
         self, indexers: Mapping[Any, int | slice | np.ndarray | Variable]
     ) -> PandasIndex | None:
-        from .variable import Variable
+        from xarray.core.variable import Variable
 
         indxr = indexers[self.dim]
         if isinstance(indxr, Variable):
@@ -429,8 +728,8 @@ class PandasIndex(Index):
     def sel(
         self, labels: dict[Any, Any], method=None, tolerance=None
     ) -> IndexSelResult:
-        from .dataarray import DataArray
-        from .variable import Variable
+        from xarray.core.dataarray import DataArray
+        from xarray.core.variable import Variable
 
         if method is not None and not isinstance(method, str):
             raise TypeError("``method`` must be a string")
@@ -497,7 +796,11 @@ class PandasIndex(Index):
             return False
         return self.index.equals(other.index) and self.dim == other.dim
 
-    def join(self: PandasIndex, other: PandasIndex, how: str = "inner") -> PandasIndex:
+    def join(
+        self,
+        other: Self,
+        how: str = "inner",
+    ) -> Self:
         if how == "outer":
             index = self.index.union(other.index)
         else:
@@ -508,7 +811,7 @@ class PandasIndex(Index):
         return type(self)(index, self.dim, coord_dtype=coord_dtype)
 
     def reindex_like(
-        self, other: PandasIndex, method=None, tolerance=None
+        self, other: Self, method=None, tolerance=None
     ) -> dict[Hashable, Any]:
         if not self.index.is_unique:
             raise ValueError(
@@ -660,12 +963,12 @@ class PandasMultiIndex(PandasIndex):
         return obj
 
     @classmethod
-    def concat(  # type: ignore[override]
+    def concat(
         cls,
-        indexes: Sequence[PandasMultiIndex],
+        indexes: Sequence[Self],
         dim: Hashable,
         positions: Iterable[Iterable[int]] | None = None,
-    ) -> PandasMultiIndex:
+    ) -> Self:
         new_pd_index = cls._concat_indexes(indexes, dim, positions)
 
         if not indexes:
@@ -814,7 +1117,7 @@ class PandasMultiIndex(PandasIndex):
     def create_variables(
         self, variables: Mapping[Any, Variable] | None = None
     ) -> IndexVars:
-        from .variable import IndexVariable
+        from xarray.core.variable import IndexVariable
 
         if variables is None:
             variables = {}
@@ -848,8 +1151,8 @@ class PandasMultiIndex(PandasIndex):
         return index_vars
 
     def sel(self, labels, method=None, tolerance=None) -> IndexSelResult:
-        from .dataarray import DataArray
-        from .variable import Variable
+        from xarray.core.dataarray import DataArray
+        from xarray.core.variable import Variable
 
         if method is not None or tolerance is not None:
             raise ValueError(
@@ -900,12 +1203,12 @@ class PandasMultiIndex(PandasIndex):
             coord_name, label = next(iter(labels.items()))
 
             if is_dict_like(label):
-                invalid_levels = [
+                invalid_levels = tuple(
                     name for name in label if name not in self.index.names
-                ]
+                )
                 if invalid_levels:
                     raise ValueError(
-                        f"invalid multi-index level names {invalid_levels}"
+                        f"multi-index level names {invalid_levels} not found in indexes {tuple(self.index.names)}"
                     )
                 return self.sel(label)
 
@@ -975,7 +1278,7 @@ class PandasMultiIndex(PandasIndex):
             # variable(s) attrs and encoding metadata are propagated
             # when replacing the indexes in the resulting xarray object
             new_vars = new_index.create_variables()
-            indexes = cast(Dict[Any, Index], {k: new_index for k in new_vars})
+            indexes = cast(dict[Any, Index], {k: new_index for k in new_vars})
 
             # add scalar variable for each dropped level
             variables = new_vars
@@ -1085,19 +1388,22 @@ T_PandasOrXarrayIndex = TypeVar("T_PandasOrXarrayIndex", Index, pd.Index)
 
 
 class Indexes(collections.abc.Mapping, Generic[T_PandasOrXarrayIndex]):
-    """Immutable proxy for Dataset or DataArrary indexes.
+    """Immutable proxy for Dataset or DataArray indexes.
 
-    Keys are coordinate names and values may correspond to either pandas or
-    xarray indexes.
+    It is a mapping where keys are coordinate names and values are either pandas
+    or xarray indexes.
 
-    Also provides some utility methods.
+    It also contains the indexed coordinate variables and provides some utility
+    methods.
 
     """
 
+    _index_type: type[Index] | type[pd.Index]
     _indexes: dict[Any, T_PandasOrXarrayIndex]
     _variables: dict[Any, Variable]
 
     __slots__ = (
+        "_index_type",
         "_indexes",
         "_variables",
         "_dims",
@@ -1108,8 +1414,9 @@ class Indexes(collections.abc.Mapping, Generic[T_PandasOrXarrayIndex]):
 
     def __init__(
         self,
-        indexes: dict[Any, T_PandasOrXarrayIndex],
-        variables: dict[Any, Variable],
+        indexes: Mapping[Any, T_PandasOrXarrayIndex] | None = None,
+        variables: Mapping[Any, Variable] | None = None,
+        index_type: type[Index] | type[pd.Index] = Index,
     ):
         """Constructor not for public consumption.
 
@@ -1118,11 +1425,33 @@ class Indexes(collections.abc.Mapping, Generic[T_PandasOrXarrayIndex]):
         indexes : dict
             Indexes held by this object.
         variables : dict
-            Indexed coordinate variables in this object.
+            Indexed coordinate variables in this object. Entries must
+            match those of `indexes`.
+        index_type : type
+            The type of all indexes, i.e., either :py:class:`xarray.indexes.Index`
+            or :py:class:`pandas.Index`.
 
         """
-        self._indexes = indexes
-        self._variables = variables
+        if indexes is None:
+            indexes = {}
+        if variables is None:
+            variables = {}
+
+        unmatched_keys = set(indexes) ^ set(variables)
+        if unmatched_keys:
+            raise ValueError(
+                f"unmatched keys found in indexes and variables: {unmatched_keys}"
+            )
+
+        if any(not isinstance(idx, index_type) for idx in indexes.values()):
+            index_type_str = f"{index_type.__module__}.{index_type.__name__}"
+            raise TypeError(
+                f"values of indexes must all be instances of {index_type_str}"
+            )
+
+        self._index_type = index_type
+        self._indexes = dict(**indexes)
+        self._variables = dict(**variables)
 
         self._dims: Mapping[Hashable, int] | None = None
         self.__coord_name_id: dict[Any, int] | None = None
@@ -1157,7 +1486,7 @@ class Indexes(collections.abc.Mapping, Generic[T_PandasOrXarrayIndex]):
 
     @property
     def dims(self) -> Mapping[Hashable, int]:
-        from .variable import calculate_dimensions
+        from xarray.core.variable import calculate_dimensions
 
         if self._dims is None:
             self._dims = calculate_dimensions(self._variables)
@@ -1237,7 +1566,7 @@ class Indexes(collections.abc.Mapping, Generic[T_PandasOrXarrayIndex]):
             A dictionary of all dimensions shared by an index.
 
         """
-        from .variable import calculate_dimensions
+        from xarray.core.variable import calculate_dimensions
 
         return calculate_dimensions(self.get_all_coords(key, errors=errors))
 
@@ -1270,10 +1599,10 @@ class Indexes(collections.abc.Mapping, Generic[T_PandasOrXarrayIndex]):
             elif isinstance(idx, Index):
                 indexes[k] = idx.to_pandas_index()
 
-        return Indexes(indexes, self._variables)
+        return Indexes(indexes, self._variables, index_type=pd.Index)
 
     def copy_indexes(
-        self, deep: bool = True, memo: dict[int, Any] | None = None
+        self, deep: bool = True, memo: dict[int, T_PandasOrXarrayIndex] | None = None
     ) -> tuple[dict[Hashable, T_PandasOrXarrayIndex], dict[Hashable, Variable]]:
         """Return a new dictionary with copies of indexes, preserving
         unique indexes.
@@ -1290,6 +1619,7 @@ class Indexes(collections.abc.Mapping, Generic[T_PandasOrXarrayIndex]):
         new_indexes = {}
         new_index_vars = {}
 
+        idx: T_PandasOrXarrayIndex
         for idx, coords in self.group_by_index():
             if isinstance(idx, pd.Index):
                 convert_new_idx = True
@@ -1325,7 +1655,8 @@ class Indexes(collections.abc.Mapping, Generic[T_PandasOrXarrayIndex]):
         return self._indexes[key]
 
     def __repr__(self):
-        return formatting.indexes_repr(self)
+        indexes = formatting._get_indexes_dict(self)
+        return formatting.indexes_repr(indexes)
 
 
 def default_indexes(
@@ -1349,7 +1680,7 @@ def default_indexes(
     coord_names = set(coords)
 
     for name, var in coords.items():
-        if name in dims:
+        if name in dims and var.ndim == 1:
             index, index_vars = create_default_index_implicit(var, coords)
             if set(index_vars) <= coord_names:
                 indexes.update({k: index for k in index_vars})
@@ -1415,6 +1746,11 @@ def indexes_all_equal(
         )
 
     indexes = [e[0] for e in elements]
+
+    same_objects = all(indexes[0] is other_idx for other_idx in indexes[1:])
+    if same_objects:
+        return True
+
     same_type = all(type(indexes[0]) is type(other_idx) for other_idx in indexes[1:])
     if same_type:
         try:
@@ -1477,7 +1813,7 @@ def filter_indexes_from_coords(
     of coordinate names.
 
     """
-    filtered_indexes: dict[Any, Index] = dict(**indexes)
+    filtered_indexes: dict[Any, Index] = dict(indexes)
 
     index_coord_names: dict[Hashable, set[Hashable]] = defaultdict(set)
     for name, idx in indexes.items():
