@@ -3,13 +3,11 @@ from __future__ import annotations
 import warnings
 from typing import TYPE_CHECKING, Literal, TypedDict
 
-from .utils import FrozenDict
+from xarray.core.utils import FrozenDict
 
 if TYPE_CHECKING:
-    try:
-        from matplotlib.colors import Colormap
-    except ImportError:
-        Colormap = str
+    from matplotlib.colors import Colormap
+
     Options = Literal[
         "arithmetic_join",
         "cmap_divergent",
@@ -106,7 +104,7 @@ _VALIDATORS = {
 
 
 def _set_file_cache_maxsize(value) -> None:
-    from ..backends.file_manager import FILE_CACHE
+    from xarray.backends.file_manager import FILE_CACHE
 
     FILE_CACHE.maxsize = value
 
@@ -164,11 +162,11 @@ class set_options:
     cmap_divergent : str or matplotlib.colors.Colormap, default: "RdBu_r"
         Colormap to use for divergent data plots. If string, must be
         matplotlib built-in colormap. Can also be a Colormap object
-        (e.g. mpl.cm.magma)
+        (e.g. mpl.colormaps["magma"])
     cmap_sequential : str or matplotlib.colors.Colormap, default: "viridis"
         Colormap to use for nondivergent data plots. If string, must be
         matplotlib built-in colormap. Can also be a Colormap object
-        (e.g. mpl.cm.magma)
+        (e.g. mpl.colormaps["magma"])
     display_expand_attrs : {"default", True, False}
         Whether to expand the attributes section for display of
         ``DataArray`` or ``Dataset`` objects. Can be
