@@ -1,15 +1,16 @@
 from __future__ import annotations
 
-from typing import TYPE_CHECKING, Generic, Hashable, Iterable, Literal, Sequence, cast
+from collections.abc import Hashable, Iterable, Sequence
+from typing import TYPE_CHECKING, Generic, Literal, cast
 
 import numpy as np
 from numpy.typing import ArrayLike
 
-from . import duck_array_ops, utils
-from .alignment import align, broadcast
-from .computation import apply_ufunc, dot
-from .pycompat import is_duck_dask_array
-from .types import Dims, T_Xarray
+from xarray.core import duck_array_ops, utils
+from xarray.core.alignment import align, broadcast
+from xarray.core.computation import apply_ufunc, dot
+from xarray.core.pycompat import is_duck_dask_array
+from xarray.core.types import Dims, T_Xarray
 
 # Weighted quantile methods are a subset of the numpy supported quantile methods.
 QUANTILE_METHODS = Literal[
@@ -125,8 +126,8 @@ _WEIGHTED_QUANTILE_DOCSTRING_TEMPLATE = """
 
 
 if TYPE_CHECKING:
-    from .dataarray import DataArray
-    from .dataset import Dataset
+    from xarray.core.dataarray import DataArray
+    from xarray.core.dataset import Dataset
 
 
 class Weighted(Generic[T_Xarray]):
@@ -162,7 +163,7 @@ class Weighted(Generic[T_Xarray]):
         Missing values can be replaced by ``weights.fillna(0)``.
         """
 
-        from .dataarray import DataArray
+        from xarray.core.dataarray import DataArray
 
         if not isinstance(weights, DataArray):
             raise ValueError("`weights` must be a DataArray")
@@ -197,10 +198,11 @@ class Weighted(Generic[T_Xarray]):
             dims = [dim] if dim else []
         else:
             dims = list(dim)
-        missing_dims = set(dims) - set(self.obj.dims) - set(self.weights.dims)
+        all_dims = set(self.obj.dims).union(set(self.weights.dims))
+        missing_dims = set(dims) - all_dims
         if missing_dims:
             raise ValueError(
-                f"{self.__class__.__name__} does not contain the dimensions: {missing_dims}"
+                f"Dimensions {tuple(missing_dims)} not found in {self.__class__.__name__} dimensions {tuple(all_dims)}"
             )
 
     @staticmethod
@@ -237,7 +239,10 @@ class Weighted(Generic[T_Xarray]):
         # (and not 2); GH4074
         if self.weights.dtype == bool:
             sum_of_weights = self._reduce(
-                mask, self.weights.astype(int), dim=dim, skipna=False
+                mask,
+                duck_array_ops.astype(self.weights, dtype=int),
+                dim=dim,
+                skipna=False,
             )
         else:
             sum_of_weights = self._reduce(mask, self.weights, dim=dim, skipna=False)
@@ -319,6 +324,7 @@ class Weighted(Generic[T_Xarray]):
         def _get_h(n: float, q: np.ndarray, method: QUANTILE_METHODS) -> np.ndarray:
             """Return the interpolation parameter."""
             # Note that options are not yet exposed in the public API.
+            h: np.ndarray
             if method == "linear":
                 h = (n - 1) * q + 1
             elif method == "interpolated_inverted_cdf":
@@ -342,7 +348,6 @@ class Weighted(Generic[T_Xarray]):
             skipna: bool,
             method: QUANTILE_METHODS = "linear",
         ) -> np.ndarray:
-
             # This algorithm has been adapted from:
             #   https://aakinshin.net/posts/weighted-quantiles/#reference-implementation
             is_nan = np.isnan(data)
@@ -443,7 +448,6 @@ class Weighted(Generic[T_Xarray]):
         return result
 
     def _implementation(self, func, dim, **kwargs):
-
         raise NotImplementedError("Use `Dataset.weighted` or `DataArray.weighted`")
 
     def sum_of_weights(
@@ -451,7 +455,6 @@ class Weighted(Generic[T_Xarray]):
         dim: Dims = None,
         keep_attrs: bool | None = None,
     ) -> T_Xarray:
-
         return self._implementation(
             self._sum_of_weights, dim=dim, keep_attrs=keep_attrs
         )
@@ -462,7 +465,6 @@ class Weighted(Generic[T_Xarray]):
         skipna: bool | None = None,
         keep_attrs: bool | None = None,
     ) -> T_Xarray:
-
         return self._implementation(
             self._sum_of_squares, dim=dim, skipna=skipna, keep_attrs=keep_attrs
         )
@@ -473,7 +475,6 @@ class Weighted(Generic[T_Xarray]):
         skipna: bool | None = None,
         keep_attrs: bool | None = None,
     ) -> T_Xarray:
-
         return self._implementation(
             self._weighted_sum, dim=dim, skipna=skipna, keep_attrs=keep_attrs
         )
@@ -484,7 +485,6 @@ class Weighted(Generic[T_Xarray]):
         skipna: bool | None = None,
         keep_attrs: bool | None = None,
     ) -> T_Xarray:
-
         return self._implementation(
             self._weighted_mean, dim=dim, skipna=skipna, keep_attrs=keep_attrs
         )
@@ -495,7 +495,6 @@ class Weighted(Generic[T_Xarray]):
         skipna: bool | None = None,
         keep_attrs: bool | None = None,
     ) -> T_Xarray:
-
         return self._implementation(
             self._weighted_var, dim=dim, skipna=skipna, keep_attrs=keep_attrs
         )
@@ -506,7 +505,6 @@ class Weighted(Generic[T_Xarray]):
         skipna: bool | None = None,
         keep_attrs: bool | None = None,
     ) -> T_Xarray:
-
         return self._implementation(
             self._weighted_std, dim=dim, skipna=skipna, keep_attrs=keep_attrs
         )
@@ -519,7 +517,6 @@ class Weighted(Generic[T_Xarray]):
         keep_attrs: bool | None = None,
         skipna: bool = True,
     ) -> T_Xarray:
-
         return self._implementation(
             self._weighted_quantile, q=q, dim=dim, skipna=skipna, keep_attrs=keep_attrs
         )
@@ -534,7 +531,6 @@ class Weighted(Generic[T_Xarray]):
 
 class DataArrayWeighted(Weighted["DataArray"]):
     def _implementation(self, func, dim, **kwargs) -> DataArray:
-
         self._check_dim(dim)
 
         dataset = self.obj._to_temp_dataset()
@@ -544,14 +540,12 @@ class DataArrayWeighted(Weighted["DataArray"]):
 
 class DatasetWeighted(Weighted["Dataset"]):
     def _implementation(self, func, dim, **kwargs) -> Dataset:
-
         self._check_dim(dim)
 
         return self.obj.map(func, dim=dim, **kwargs)
 
 
 def _inject_docstring(cls, cls_name):
-
     cls.sum_of_weights.__doc__ = _SUM_OF_WEIGHTS_DOCSTRING.format(cls=cls_name)
 
     cls.sum.__doc__ = _WEIGHTED_REDUCE_DOCSTRING_TEMPLATE.format(
